%{
#include <string.h>
#include "dynare2.h"
#include "d_tab.hh"
int previous_state;
int comment_level = 0;
int lineno = 1;
void yyerror(char *);
%}



%option stack
%option yylineno
%x CMT

%%



 /* literal keywords token */

endval {return ENDVAL;}	       
initval {return INITVAL;}
model {return MODEL;}
periods {return PERIODS;}
shocks {return SHOCKS;}
mshocks {return MSHOCKS;}
var {return VAR;}
varexo {return VAREXO;}
vardef {return VARDEF;}
parameters {return PARAMETERS;}
dyn2vec {return DYN2VEC;}
dyn2gau {return DYN2VEC;}
rplot {return RPLOT;}
steady {return STEADY;}
vcov {return VCOV;}
order {return ORDER;}

 /* comments */
<*>"/*" {yy_push_state(CMT);}
<CMT>[^*\n]*
<CMT>"*"+[^*/\n]*
<CMT>"*"+"/" {yy_pop_state();}
<CMT>[\n]
"//".*\n

 /* initial spaces or tabs are ignored */

[ \t\n] ;
values {return VALUES;}
[+] {return PLUS;}
[-] {return MINUS;}
[*] {return TIMES;}
[/] {return DIVIDE;}
[\^] {return POWER;}
[!] {return FACTORIAL;}
exp {return EXP;}
log {return LOG;}
log10 {return LOG10;}
ln {return LN;}
sin {return SIN;}
cos {return COS;}
tan {return TAN;}
asin {return ASIN;}
acos {return ACOS;}
atan {return ATAN;}
sinh {return SINH;}
cosh {return COSH;}
tanh {return TANH;}
asinh {return ASINH;}
acosh {return ACOSH;}
atanh {return ATANH;}
sqrt  {return SQRT;}

end {return END;}


 /* do loops in models */
do {return DO;}
to {return TO;}
by {return BY;}
endo {return ENDO;}
sum {return SUM;}
prod {return PROD;}

 /* shocks statements */
stderr {return STDERR;}

[A-Za-z_][A-Za-z0-9_]* {
  yylval.string=strdup(yytext);
  return NAME;}


 /* floating-point numbers */
(([0-9]*\.[0-9]+)|([0-9]+\.))([edED][-+]?[0-9]+)? {
  yylval.string=strdup(yytext);
  return DNUMBER;}

 /* integer number */
[0-9]+ {
  yylval.string=strdup(yytext);
  return INUMBER;}
 
= {return EQUAL_SIGN;}
\$ {return DOLLAR;}

. {return yytext[0];}

%%

int yywrap()
{
 return 1;
}






