#ifndef DYNARE2_H
#define DYNARE2_H

#include "../tl/cc/int_sequence.h"
#include "../tl/cc/t_container.h"
#include "../tl/cc/sparse_tensor.h"
#include "../tl/cc/twod_matrix.h"

#include "../sylv/cc/Vector.h"

#include <sstream>
#include <fstream>
#include <vector>
#include <deque>
#include <list>
#include <map>
#include <ext/hash_map>
#include <cstdio>
#include <cstring>
#include <string>
#include <algorithm>

typedef std::vector<int> vec_int;
typedef std::vector<double> vec_double;

struct state_elt
{
  int nbr;
  int lag;
  int d_col;
  int e_col;
  const char* name;
};

typedef std::vector<state_elt> t_state;

struct model_struct {
	int eq_nbr;
	int exo_nbr;
	int endo_nbr;
	int param_nbr;
	int dyn_endo_nbr;
	int fwrd_nbr;
	int s_fwrd_nbr;
	int static_nbr;
	int lagged_nbr;
	int s_lagged_nbr;
	int p_static;
	int p_c_lagged;
	int p_c_fwrd;
	int p_fwrd;
	int p_lagged;
	int p_exo;
	int c_lagged_nbr;
	int c_fwrd_nbr;
	int c_both_nbr;
	int maxlag;
	int maxlead;
	t_state state_vec;

	model_struct() {
		eq_nbr = 0; exo_nbr = 0; endo_nbr = 0; param_nbr = 0; dyn_endo_nbr = 0; fwrd_nbr = 0;
		s_fwrd_nbr = 0; static_nbr = 0; lagged_nbr = 0; s_lagged_nbr = 0;
		p_static = 0; p_c_lagged = 0; p_c_fwrd = 0; p_fwrd = 0; p_lagged = 0;
		p_exo = 0; c_lagged_nbr = 0; c_fwrd_nbr = 0; c_both_nbr = 0;
		maxlag = 0; maxlead = 0;
	}
		
};

//typedef std::_List_node_base* t_my_ref;
typedef struct s_node * t_my_ref;

struct s_symbol {
  int type;
  int nbr;
};

struct s_variable_key {
  const char *name;
  int lag;
};

using __gnu_cxx::hash;
using __gnu_cxx::hash_map;

typedef hash_map< int, t_my_ref, hash<int> > t_derivatives; 

struct s_node {
	int code;
	int ref;
	int no;
	t_my_ref arg1;
	t_my_ref arg2;
	double result;
	t_derivatives derivatives;
	void print() const {
		printf("%p: code=%d, ref=%d, no=%d, arg1=%p, arg2=%p\n",
			   this, code, ref, no, arg1, arg2);
	}
};

struct s_variable_data {
  int type;
  int nbr;
  t_my_ref address;
};

struct eqstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) == 0;
  }
};

struct s_deriv{
  int var;
  int col;
  int deriv_order;
  IntSequence intseq;
  s_deriv(int v, int c, int o, const IntSequence& s)
    : var(v), col(c), deriv_order(o), intseq(s) {}
};

struct variable_comp
{
  bool operator()(const struct s_variable_key & s1, const struct s_variable_key & s2);
};

struct if_no_ref : public std::unary_function<s_node, bool>
{
  bool operator()(s_node s);
};

struct s_mixed{
  int nbr;
  int id;
  int mix;
};

typedef std::map< const struct s_variable_key, struct s_variable_data, variable_comp> type_variable_table; 
typedef hash_map< const char*, struct s_symbol, hash< const char*>, eqstr> type_symbol_table; 
typedef hash_map< const int, struct s_deriv, hash<int> > t_h_deriv;
typedef std::list< s_node > type_model;
typedef std::list<s_node>::iterator tm_it;
typedef std::list<s_node>::reverse_iterator tm_rit;
typedef std::vector<type_variable_table::iterator> t_var_deriv_index;
int yyparse();

#endif





