#include "dynare2.h"
#include "dynare3.h"

double tolx = 3.7e-11;


void Dynare::nlsolve(Vector & x)
{
  Vector fvec(sm.endo_nbr), p(sm.endo_nbr);
  vec_double junk1;
  vec_int ptr(sm.endo_nbr), junk2, junk3;

  x.print();
  for(int i = 0; i < -sm.maxlag+sm.maxlead+1; i++) {
	  Vector ri(i, *y);
      ri = (const Vector&)x;
  }
  double stpmax = 100*std::max(x.getNorm(), sqrt((double)x.length()));
  int it = 1-sm.maxlag;
  execute_tree(md, fvec, junk1, junk2, junk3, end_resid, it, 0);
  std::cout << "fvec\n";
  fvec.print();

  double f = 0.5*fvec.getNorm()*fvec.getNorm();
  if (isnan(f)) {
      std::cout << "f is NaN\n";
      exit(1);
  }

  while (1) {
      double err = fvec.getMax();
      std::cout << "Err: " << err << "\n";
      if (err < 1e-10)
		  break;

      for(int i = 0; i < -sm.maxlag+sm.maxlead+1; i++) {
		  Vector ri(i, *y);
		  ri = (const Vector&)x;
	  }
      TwoDMatrix Jacobian(sm.endo_nbr,sm.endo_nbr);
	  Jacobian.zeros();
      jacobian(fvec, Jacobian);
//       std::cout << "Jacobian\n";
//       print_all_matrix(Jacobian);
      Vector g(sm.endo_nbr);
	  g.zeros();
	  Jacobian.multaVecTrans(g, fvec);
	  Vector p(fvec);
	  ConstTwoDMatrix(Jacobian).multInvLeft(p);
	  p.mult(-1.0);
      lnsrch(x, f, g, p, stpmax, fvec);
    }
}

int Dynare::lnsrch(Vector & x, double f, Vector & g, Vector & p,
				   double stpmax, Vector & fvec)
{
	double alf=1e-4;
	double fold = f;

	double sum = p.getNorm();
	if ( sum > stpmax) {
		p.mult(stpmax/sum);
	}
  
	double slope=0.0;
	for (int i = 0; i < g.length(); i++)
		slope += g[i]*p[i];


	double test = 0;
	for (int i = 0; i < p.length(); ++i) {
		double tmp = fabs(p[i]) / ((fabs(x[i]) > 1) ? x[i] : 1); 
		if ( tmp > test )
			test = tmp;
	}
	double alamin = tolx / test;

	double alam = 1;
	while (1) {
		if (alam < alamin)
			return 1;
	
		x.add(alam, p);
		for(int i = 0; i < -sm.maxlag+sm.maxlead+1; i++) {
			Vector ri(i, *y);
			ri = (const Vector&) x;
		}
		vec_double junk1;
		vec_int junk2, junk3;
		execute_tree(md, fvec, junk1, junk2, junk3, end_resid, 1-sm.maxlag, 0);
		double f =0.5*fvec.getNorm()*fvec.getNorm();
		if (isnan(f)) {
			std::cerr << "f is NaN\n";
			fvec.print();
			exit(1);
		}

		if (f <= fold+alf*alam*slope) {
			std::cout << "Alam " << alam << "\n";
			return 0;
		}
		double tmplam, f2=f, fold2=fold, alam2=alam;
		if (alam == 1) {
			tmplam = -slope/(2*(f-fold-slope));
		} else {
			double rhs1 = f-fold-alam*slope;
			double rhs2 = f2-fold2-alam2*slope;
			double a = (rhs1/(alam*alam)-rhs2/(alam2*alam2))/(alam-alam2);
			double b=(-alam2*rhs1/(alam*alam)+alam*rhs2/(alam2*alam2))/(alam-alam2);
			if (a == 0) {
				tmplam = -slope/(2*b);
			} else {
				double disc = b*b-3*a*slope;
				if (disc < 0) {
					std::cerr << "ERROR: Roundoff problem in nlsearch\n";
					exit(1);
				} else {
					tmplam=(-b+sqrt(disc))/(3*a);
				}
			}
			if (tmplam > 0.5*alam)
				tmplam = 0.5*alam;
		}
		alam2=alam;
		f2=f;
		fold2=fold;
		alam=std::max(tmplam,0.1*alam);
	}
}
