// function state computes the composition of the state vector z(t+1) 
// such that Dz(t+1) = Ez(t) in the solution of the first order approximation
// state_vec is dense1D vector of strucures
// int nbr: id of reordered endogenous variable
// int lag: lag (negative) or lead (positive) of variable in z(t+1)
// int d_col: index of column of C matrix (computed in first_order.cc)
//            for elements of D matrix above (starts at 0)
// int e_col: index of column of C matrix (computed in first_order.cc)
//            for elements of E matrix above (starts at 0)
// Note at lag == +1, a 1 in e_col means that the corresponding elements of E
//                    must be set to -1
// Note at lag == 0, a 1 in d_col means that the corresponding elements of D
//                    must be set to 1

#include "dynare2.h"
#include "dynare3.h"

void Dynare::state(t_state& state_vec)
{
  vec_int v1(sm.endo_nbr), v2(sm.endo_nbr);
  t_state s1;

  // leads from maxlead down
  t_var_deriv_index::iterator iv=var_deriv_index.begin();
  for(iv = var_deriv_index.begin(); iv != var_deriv_index.end(); ++iv)
    {
      std::cout << ((*iv)->first).name << " " << ((*iv)->first).lag << " " << ((*iv)->second).nbr << "\n";
    }
  iv=var_deriv_index.begin();
  int k2 = 0;
  while ( iv != var_deriv_index.end()&& ((*iv)->second).type != 1 )
    {
      ++iv;
      ++k2;
    }

  sm.s_fwrd_nbr = 0;
  for(int i=sm.maxlead ; i > 1; i--) 
    {
      for(int j=0; j < sm.endo_nbr; j++)
	{
	  if ( ((*iv)->second).nbr == j && ((*iv)->first).lag == i)
	    {
	      state_elt se;
	      se.nbr = j;
	      se.lag = i;
	      se.d_col = k2;
	      se.e_col = -1;
	      se.name = ((*iv)->first).name;
	      state_vec.push_back(se);
	      v2[j] = 1;
	      ++iv;
	      ++k2;
	      while (  iv != var_deriv_index.end() && ((*iv)->second).type != 1 )
		{
		  ++iv;
		  ++k2;
		}
	      ++sm.s_fwrd_nbr;
	    }
	  else if (v2[j] == 1)
	    {
	      state_elt se;
	      se.nbr = j;
	      se.lag = i;
	      se.d_col = -1;
	      se.e_col = -1;
	      //	      se.name = ((*iv)->first).name;
	      se.name = "";
	      state_vec.push_back(se);
	      ++sm.s_fwrd_nbr;
	    }
	}
    }

  t_var_deriv_index::reverse_iterator riv=var_deriv_index.rbegin();
  int k1 = var_deriv_index.size()-1;
  while (  riv != var_deriv_index.rend() && ((*riv)->second).type != 1 )
    {
      ++riv;
      --k1;
    }
  for(int i=sm.maxlag ; i < 0; i++) 
    {
      for(int j=sm.endo_nbr-1; j >= 0; j--)
	{
	  if ( ((*riv)->second).nbr == j && ((*riv)->first).lag == i)
	    {
	      state_elt se;
	      se.nbr = j;
	      se.lag = i+1;
	      se.d_col = -1;
	      se.e_col = k1-sm.endo_nbr;
	      se.name = ((*riv)->first).name;
	      s1.push_back(se);
	      v1[j] = 1;
	      ++riv;
	      --k1;
	      while ( riv != var_deriv_index.rend() && ((*riv)->second).type != 1 )
		{
		  ++riv;
		  --k1;
		}
	    }
	  else if (v1[j] == 1)
	    {
	      state_elt se;
	      se.nbr = j;
	      se.lag = i+1;
	      se.d_col = -1;
	      se.e_col = -1;
	      //	      se.name = ((*riv)->first).name;
	      se.name = "";
	      s1.push_back(se);
	    }
	}
    }

  // current variables; compares variables with -1 lag (iv) 
  // and current variables (iv1)
  t_var_deriv_index::iterator iv1=iv;
  int k3 = k2;
  while (  iv1 != var_deriv_index.end() && (((*iv1)->first).lag > 0 || ((*iv1)->second).type != 1))
    {
      ++k3;
      ++iv1;
    }
  std::cout << "k3 " << k3 << "\n";
  // current value of lead variable
  for(int j = 0; j < sm.endo_nbr; j++)
    {
      state_elt se;
      se.lag = 1;
      se.nbr = j;
      if ( ((*iv)->second).nbr == j && ((*iv)->first).lag == 1)
	{
	  se.d_col = k2;
	  se.e_col = 1;
	  se.name = ((*iv)->first).name;
	  state_vec.push_back(se);
	  v2[j] = 1;
	  ++sm.s_fwrd_nbr;
	  ++k2;
	  ++iv;
	  while (  iv != var_deriv_index.end() && ((*iv)->second).type != 1 )
	    {
	      ++iv;
	      ++k2;
	    }
	}
      else if (v2[j] == 1)
	{
	  se.d_col = -1;
	  se.e_col = 1;
	  //	  se.name = ((*riv)->first).name;
	  se.name = "";
	  state_vec.push_back(se);
	  ++sm.s_fwrd_nbr;
	}
    }

  // add current value of laggedd variables to state vector
  t_state::reverse_iterator ris = s1.rbegin();
  for (int j = 0; j < sm.endo_nbr; j++)
    {
      if (v1[j] == 1)
	{
	  if (v2[j] == 0)
	    {
	      (*ris).d_col = 1;
	    }
	  state_vec.push_back(*ris);
	  ++ris;
	}
    }
  
  // add remaining lagged variables to state vector
  while ( ris != s1.rend() )
    {
      state_vec.push_back(*ris);
      ++ris;
    }

  sm.s_lagged_nbr = state_vec.size() - sm.s_fwrd_nbr;

}
